using System.Collections.Concurrent;

namespace NSeguin.Dev.Web.Components.Shared.HeroIcons;

internal readonly record struct HeroIconParameters(HeroIconKind Kind, HeroIconVariant Variant)
{
    private const string PathBase = "wwwroot/lib/heroicons";
    private static readonly ConcurrentDictionary<HeroIconParameters, string> SvgCache = new();

    public Task<string> GetSvgAsync()
    {
        if (SvgCache.TryGetValue(this, out string? svg))
        {
            return Task.FromResult(svg);
        }

        return LoadSvgAsync();
    }

    private static string GetPath(HeroIconVariant variant, HeroIconKind kind)
    {
        string pathSuffix = variant switch
        {
            HeroIconVariant.Solid => "24/solid",
            HeroIconVariant.Outline => "24/outline",
            HeroIconVariant.Small => "20/solid",
            HeroIconVariant.ExtraSmall => "16/solid",
            _ => throw new ArgumentOutOfRangeException(nameof(variant))
        };

        return Path.Combine(PathBase, pathSuffix, kind.FileName);
    }

    private string GetPath()
    {
        return GetPath(Variant, Kind);
    }

    private async Task<string> LoadSvgAsync()
    {
        string path = GetPath();
        string svg = await File.ReadAllTextAsync(path);
        SvgCache.TryAdd(this, svg);
        return svg;
    }
}
